@@ -1,9 +1,9 @@
 //! Module for parsing and handling place notation
 
-use crate::{AnnotBlock, AnnotRow, Bell, IncompatibleStages, Row, RowBuf, Stage};
+use crate::{AnnotBlock, Bell, IncompatibleStages, Row, RowBuf, SameStageVec, Stage};
 use itertools::Itertools;
 use std::{
-    fmt::{Display, Formatter},
+    fmt::{Debug, Display, Formatter},
     ops::Range,
 };
 
@@ -15,14 +15,15 @@ use crate::Block;
 pub enum ParseError {
     PlaceOutOfStage { place: usize, stage: Stage },
     AmbiguousPlacesBetween { p: usize, q: usize },
-    OddStageCross { stage: Stage },
+    DuplicatePlace(usize),
+    OddStageCross(Stage),
     NoPlacesGiven,
 }
 
 impl Display for ParseError {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         match self {
-            ParseError::OddStageCross { stage } => {
+            ParseError::OddStageCross(stage) => {
                 write!(f, "Cross notation for odd stage {}", stage)
             }
             ParseError::PlaceOutOfStage { place, stage } => {
@@ -43,12 +44,15 @@ impl Display for ParseError {
             ParseError::NoPlacesGiven => {
                 write!(f, "No places given.  Use 'x' or '-' for a cross.")
             }
+            ParseError::DuplicatePlace(p) => {
+                write!(f, "Place '{}' is duplicated", Bell::from_index(*p))
+            }
         }
     }
 }
 
 /// A single piece of place notation on any [`Stage`].
-#[derive(Debug, Clone, Eq, PartialEq, Hash)]
+#[derive(Clone, Eq, PartialEq, Hash)]
 pub struct PlaceNot {
     /// A **0-indexed** list of which places are made during this `PlaceNot`.  We maintain the
     /// following invariants:
@@ -58,14 +62,13 @@ pub struct PlaceNot {
     ///   For example, suppose the [`Stage`] is [`MAJOR`](Stage::MAJOR).
     ///    - The place notation "4" has an implicit place made at lead and so would be stored as
     ///      `vec![0, 3]`.
-    ///    - The place notation "146" has an implicit/ambiguous place made in 5ths, so would be
+    ///    - The place notation "146" has an implicit place made in 5ths, so would be
     ///      stored as `vec![0, 3, 4, 5]`.
     /// - The places are stored **in ascending order**.  So "4817" would be stored as
     ///   `vec![0, 3, 6, 7]`.
     ///
-    /// Enforcing these invariants improves the the speed of permutation and equality tests at the
-    /// cost of (slightly) slower parsing, but I think this trade-off is justified since all
-    /// `PlaceNot`s are parsed only once but permuting/equality tests happen many many times.
+    /// Enforcing these invariants improves the speed of permutation and equality tests at the cost
+    /// of (slightly) slower parsing.
     places: Vec<usize>,
     /// The [`Stage`] that this `PlaceNot` is intended to be used for.
     stage: Stage,
@@ -74,10 +77,10 @@ pub struct PlaceNot {
 impl PlaceNot {
     /// Parse a string, interpreting it as a single `PlaceNot` of a given [`Stage`].  Like
     /// [`RowBuf::parse_with_stage`], this ignores chars that don't correspond to valid [`Bell`]
-    /// names, including `&`, `.`, `,` and `+` which have reserved meanings in blocks of place
-    /// notation.  This will expand implicit places (even between two written places) but will fail
-    /// if there is any kind of ambiguity, returning a [`ParseError`] describing the problem.  This
-    /// also runs in `O(n)` time except for sorting the places which takes `O(n log n)` time.
+    /// names, including `&`, `.`, `,` and `+` (even though they have reserved meanings in blocks
+    /// of place notation).  This will expand implicit places (even between two written places) but
+    /// will fail if there is any kind of ambiguity, returning a [`ParseError`] describing the
+    /// problem.
     ///
     /// # Example
     /// ```
@@ -104,7 +107,7 @@ impl PlaceNot {
     pub fn parse(s: &str, stage: Stage) -> Result<Self, ParseError> {
         // If the string is any one of the cross strings, then return CROSS
         if s.len() == 1 && s.chars().next().map(CharMeaning::from) == Some(CharMeaning::Cross) {
-            return Self::cross(stage).ok_or(ParseError::OddStageCross { stage });
+            return Self::cross(stage).ok_or(ParseError::OddStageCross(stage));
         }
         // Parse the string into bell indices, ignoring any invalid characters
         let mut parsed_places: Vec<usize> = s
@@ -133,7 +136,8 @@ impl PlaceNot {
         // instead)
         parsed_places.last().ok_or(ParseError::NoPlacesGiven)?;
         // Check if any of the bells are out of range
-        if let Some(out_of_range_place) = parsed_places.last().filter(|p| **p >= stage.as_usize()) {
+        if let Some(out_of_range_place) = parsed_places.last().filter(|p| **p >= stage.num_bells())
+        {
             return Err(ParseError::PlaceOutOfStage {
                 place: *out_of_range_place,
                 stage,
@@ -152,7 +156,9 @@ impl PlaceNot {
             places.push(p);
             // Check if there is an implicit place made between these, or if the place notation is
             // ambiguous
-            let num_intermediate_places = q - p - 1;
+            let num_intermediate_places = (q - p)
+                .checked_sub(1) // If p == q, then `p - q == 0` and this subtraction will underflow ...
+                .ok_or(ParseError::DuplicatePlace(p))?; // ... so report the duplicate places
             if num_intermediate_places == 1 {
                 places.push(p + 1);
             } else if num_intermediate_places % 2 == 1 {
@@ -170,10 +176,10 @@ impl PlaceNot {
         // Add implicit place at the back if necessary
         if parsed_places
             .last()
-            .filter(|p| (stage.as_usize() - *p) % 2 == 0)
+            .filter(|p| (stage.num_bells() - *p) % 2 == 0)
             .is_some()
         {
-            places.push(stage.as_usize() - 1)
+            places.push(stage.num_bells() - 1)
         }
         // Create struct and return.  We don't need to sort `places`, because we only pushed to it
         // in ascending order.
@@ -195,7 +201,7 @@ impl PlaceNot {
     /// # Ok::<(), bellframe::place_not::ParseError>(())
     /// ```
     pub fn cross(stage: Stage) -> Option<Self> {
-        if stage.as_usize() % 2 == 0 {
+        if stage.num_bells() % 2 == 0 {
             Some(PlaceNot {
                 places: Vec::new(),
                 stage,
@@ -230,12 +236,61 @@ impl PlaceNot {
         self.places.contains(&place)
     }
 
+    /// Checks whether internal places are made in this `PlaceNot`
+    #[inline(always)]
+    pub fn has_internal_places(&self) -> bool {
+        let has_lead = self.places.first() == Some(&0);
+        let has_lie = self.places.last() == Some(&(self.stage.num_bells() - 1));
+        let num_external_places = if has_lead { 1 } else { 0 } + if has_lie { 1 } else { 0 };
+        self.places.len() > num_external_places
+    }
+
     /// Returns the [`Stage`] of this `PlaceNot`
     #[inline(always)]
     pub fn stage(&self) -> Stage {
         self.stage
     }
 
+    /// Returns the [`PlaceNot`] that goes between two [`Row`]s, or `None` if the [`Row`]s are not
+    /// adjacent.
+    pub fn pn_between(r1: &Row, r2: &Row) -> Option<PlaceNot> {
+        if r1.stage() != r2.stage() {
+            return None;
+        }
+
+        let mut places = Vec::<usize>::new();
+        let mut bell_pair_iter = r1.bell_iter().zip_eq(r2.bell_iter()).enumerate();
+        while let Some((place, (b1, b2))) = bell_pair_iter.next() {
+            // If b1 and b2 are the same, then a place must be made
+            if b1 == b2 {
+                places.push(place);
+            } else {
+                // Otherwise, we expect b1 and b2 to swap round in the next place:
+                // ... b1  b2 ...
+                //      (to)
+                // ... b2  b1 ...
+                if Some((place + 1, (b2, b1))) != bell_pair_iter.next() {
+                    return None;
+                }
+            }
+        }
+
+        Some(Self {
+            places,
+            stage: r1.stage(),
+        })
+    }
+
+    /// Returns a [`RowBuf`] representing the same transposition as this `PlaceNot`.
+    pub fn transposition(&self) -> RowBuf {
+        let mut row = RowBuf::rounds(self.stage());
+        // This unsafety is OK because the `row` has the same stage as `self`
+        unsafe {
+            self.permute_unchecked(&mut row);
+        }
+        row
+    }
+
     /// Uses this `PlaceNot` to perform an in-place permutation of a given [`Row`].  If you want to
     /// to preserve the old [`Row`], then use [`permute_new`](Self::permute_new).
     pub fn permute(&self, row: &mut Row) -> Result<(), IncompatibleStages> {
@@ -254,10 +309,10 @@ impl PlaceNot {
     pub unsafe fn permute_unchecked(&self, row: &mut Row) {
         let mut places = self.places.iter().copied().peekable();
         let mut i = 0;
-        while i < self.stage.as_usize() {
+        while i < self.stage.num_bells() {
             if places.peek() == Some(&i) {
-                // If this PN contains a place at this index, then no modification is necessary but
-                // we do need to consume the place so we move on the iterator
+                // If this PN contains a place at this index, then the bell in this place stays
+                // where it is, so no change is required.
                 places.next();
                 i += 1;
             } else {
@@ -289,6 +344,12 @@ impl PlaceNot {
     }
 }
 
+impl Debug for PlaceNot {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        write!(f, "PlaceNot({})", self)
+    }
+}
+
 impl Display for PlaceNot {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         if self.is_cross() {
@@ -296,14 +357,10 @@ impl Display for PlaceNot {
             write!(f, "-")
         } else {
             // Otherwise concatenate all the bell names together
-            write!(
-                f,
-                "{}",
-                self.places
-                    .iter()
-                    .map(|&x| Bell::from_index(x).name())
-                    .join("")
-            )
+            for p in &self.places {
+                write!(f, "{}", Bell::from_index(*p))?;
+            }
+            Ok(())
         }
     }
 }
@@ -395,7 +452,7 @@ impl PnBlock {
 
             // Handle the output of parsing the current block
             if is_single_block || is_asymmetric {
-                pns.extend(sym_block_buf.drain(..));
+                pns.append(&mut sym_block_buf);
             } else {
                 // Clone sym_block_buf into `pns` in order
                 pns.extend_from_slice(&sym_block_buf);
@@ -497,7 +554,7 @@ impl PnBlock {
             if m == CharMeaning::Cross {
                 buf.push(
                     PlaceNot::cross(stage)
-                        .ok_or(ParseError::OddStageCross { stage })
+                        .ok_or(ParseError::OddStageCross(stage))
                         .map_err(|e| PnBlockParseError::PnError(index..index + 1, e))?,
                 );
             }
@@ -567,23 +624,45 @@ impl PnBlock {
         self.pns.len()
     }
 
-    /// Generates a [`Block`] specified by these [`PlaceNot`]s.  This [`Block`] will contain only
-    /// default annotations.
-    pub fn to_block<A: Default>(&self) -> AnnotBlock<A> {
-        // The rows which will make up the new Block
-        let mut rows: Vec<AnnotRow<A>> = Vec::with_capacity(self.pns.len() + 1);
-        rows.push(AnnotRow::with_default(RowBuf::rounds(self.stage())));
+    /// Generates the [`Row`]s which follow from applying `self` to a given [`Row`].  The resulting
+    /// [`SameStageVec`] has length one greater than that of `self`, because it starts with
+    /// `start_row` and then adds one new [`Row`] per [`PlaceNot`] in `self`.
+    pub fn to_rows(&self, start_row: RowBuf) -> Result<SameStageVec, IncompatibleStages> {
+        IncompatibleStages::test_err(start_row.stage(), self.stage())?;
+
+        let mut rows = SameStageVec::from_row_buf(start_row.clone());
+        let mut current_row = start_row;
         for pn in &self.pns {
-            rows.push(AnnotRow::with_default(unsafe {
-                pn.permute_new_unchecked(&rows.last().unwrap().row())
-            }));
+            // This unsafety is OK because:
+            // - all PlaceNots in `self` have the same stage (by invariant)
+            // - we return with an error if `start_row` has a different `stage` to this `PnBlock`
+            // => `pn.stage() == current_row.stage()`
+            unsafe { pn.permute_unchecked(&mut current_row) };
+            rows.push(&current_row).unwrap(); // Unwrap is fine, because we've already checked that
+                                              // the stages match
         }
-        // This unsafety is OK, because:
-        // - rows.len() >= 2, because it contains one copy of `start_row` and one Row per PN in
-        //   this Block (and PnBlocks must have at least one PlaceNot)
-        // - These place notations must all have the same stage, so therefore the resulting Rows
-        //   must too
-        unsafe { AnnotBlock::from_annot_rows_unchecked(rows) }
+        Ok(rows)
+    }
+
+    /// Generates the [`AnnotBlock`] of this `PnBlock`, starting at some `start_row`.  All the
+    /// annotations are generated by `A::default()`.
+    pub fn to_block<A>(&self, start_row: RowBuf) -> Result<AnnotBlock<A>, IncompatibleStages>
+    where
+        A: Default,
+    {
+        let rows = self.to_rows(start_row)?;
+        Ok(AnnotBlock::with_default_annots(rows))
+    }
+
+    /// Generates the [`AnnotBlock`] of this `PnBlock`, starting at [rounds](RowBuf::rounds).  All
+    /// the annotations are generated by `A::default()`.
+    pub fn to_block_from_rounds<A>(&self) -> AnnotBlock<A>
+    where
+        A: Default,
+    {
+        // Unwrapping here is fine, because we provide a `RowBuf` that has the same `Stage` as
+        // `self`
+        self.to_block(RowBuf::rounds(self.stage())).unwrap()
     }
 }
 
@@ -614,7 +693,7 @@ impl From<char> for CharMeaning {
 #[cfg(test)]
 mod tests {
     use super::ParseError;
-    use crate::{Block, PlaceNot, PnBlock, Stage};
+    use crate::{Block, PlaceNot, PnBlock, RowBuf, Stage};
 
     #[test]
     fn parse_ok() {
@@ -675,7 +754,7 @@ mod tests {
             for cross_not in &["x", "X", "-"] {
                 assert_eq!(
                     PlaceNot::parse(*cross_not, stage),
-                    Err(ParseError::OddStageCross { stage })
+                    Err(ParseError::OddStageCross(stage))
                 );
             }
         }
@@ -697,9 +776,9 @@ mod tests {
 
     #[test]
     fn parse_err_no_places_given() {
-        for stage in 0..12 {
+        for num_bells in 1..12 {
             assert_eq!(
-                PlaceNot::parse("", Stage::from(stage)),
+                PlaceNot::parse("", Stage::new(num_bells)),
                 Err(ParseError::NoPlacesGiven)
             );
         }
@@ -796,11 +875,29 @@ mod tests {
             (Stage::MAJOR, "x18x18x18x18,12", plain_bob_major_lead),   // Plain Bob Major
         ];
 
-        for &(stage, pn, block) in &equal_blocks {
+        for &(stage, pn, block_str) in &equal_blocks {
             println!("Parsing {}", pn);
-            let b1: Block = PnBlock::parse(pn, stage).unwrap().to_block();
-            let b2 = Block::parse(block).unwrap();
-            assert_eq!(b1, b2);
+            let expected_block = Block::parse(block_str).unwrap();
+            let b1: Block = PnBlock::parse(pn, stage)
+                .unwrap()
+                .to_block(expected_block.first_annot_row().unwrap().row().to_owned())
+                .unwrap();
+            assert_eq!(b1, expected_block);
         }
     }
+
+    #[test]
+    fn pn_between_rows() {
+        #[track_caller]
+        fn check_ok(r1: &str, r2: &str, exp_pn: &str) {
+            let row1 = RowBuf::parse(r1).unwrap();
+            let row2 = RowBuf::parse(r2).unwrap();
+            let exp_pn = PlaceNot::parse(exp_pn, row1.stage()).unwrap();
+            assert_eq!(PlaceNot::pn_between(&row1, &row2), Some(exp_pn));
+        }
+
+        check_ok("12345678", "13245678", "145678");
+        check_ok("12345678", "13246587", "14");
+        check_ok("18247653", "81246753", "3478");
+    }
 }
